#ifndef MAPAGENT_QOS_INTERNAL_H
#define MAPAGENT_QOS_INTERNAL_H

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <net/if_arp.h>

#include <linux/if_bridge.h>

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif
#include <json-c/json.h>
#include <libubox/blobmsg.h>
#include <libubox/blobmsg_json.h>
#include <libubox/uloop.h>
#include <libubox/ustream.h>
#include <libubox/utils.h>
#include <libubus.h>

#if (EASYMESH_VERSION >2)
#include <openssl/err.h>
#include <openssl/bn.h>
#include <openssl/evp.h>
#include <openssl/x509.h>
#include <openssl/ec.h>
#endif

#include <cmdu.h>
#include <1905_tlvs.h>
#include <easymesh.h>
#include <i1905_wsc.h>
#include <map_module.h>

#include <uci.h>

#include "timer.h"
#include "utils/1905_ubus.h"
#include "utils/utils.h"
#include "utils/debug.h"
#include "utils/liblist.h"
#include "steer_rules.h"
#if (EASYMESH_VERSION > 2)
#include "dpp.h"
#endif
#include "config.h"
#include "nl.h"
#include "agent.h"
#include "qos.h"

#if (EASYMESH_VERSION > 2)

/* Rule types */
enum qos_rule_type {
	QOS_RULE_TYPE_DSCP_PCP,/**< DSCP policy type */
	QOS_RULE_TYPE_MSCS,    /**< MSCS rule */
	QOS_RULE_TYPE_SCS,     /**< SCS rule */
	QOS_RULE_TYPE_MGMT,    /**< Management rule */
};

/* Rule structure which contains both SPR and DSCP parts */
struct qos_rule {
	struct list_head        list;  /**< List head */
	struct tlv_spr          spr;   /**< Service Prioritization Rule */
	enum   qos_rule_type    type;  /**< Rule Type */
	union {
		struct tlv_dscp_pcp dscp;  /**< DSCP rule - valid only in case of
		                                QOS_RULE_TYPE_DSCP_PCP rule type */
	};
};

/* DSCP-PCP -> hostapd format conversion results */
enum dscp_pcp_conv_result {
	DSCP_PCP_CONV_RESULT_OK,               /**< Conversion finished well */
	DSCP_PCP_CONV_RESULT_OK_TOO_MANY_EXC,  /**< Conversion finished well,
	                                            but too many exceptions */
	DSCP_PCP_CONV_RESULT_TOO_LONG,         /**< Too long result that didn't
	                                            fit the output buffer */
	DSCP_PCP_CONV_RESULT_FAIL,             /**< Failure producing QoS map */
};

#endif

#endif
